
import numpy as np
from classy_sz import Class as Class_sz
import os
import time


cosmo_params_with_sigma_8 = {'H0': 67.36, 'ns': 0.9649, 
                             'Omega_b': 0.04, 
                             'omega_cdm': 0.1407, 'sigma8': 0.8, 'tau': 0.0544}



classy_sz = Class_sz()
classy_sz.set(cosmo_params_with_sigma_8)
classy_sz.set(
{'output': 'pCl lCl lCl',}
)
classy_sz.compute_class_szfast()

print('sigma8:',classy_sz.sigma8())
print('A_s:',classy_sz.get_current_derived_parameters(['A_s']))
print('m_ncdm_in_eV:',classy_sz.get_current_derived_parameters(['m_ncdm_in_eV']))




from classy_sz import Class as Class_sz 
import matplotlib
import matplotlib.pyplot as plt
import numpy as np
import time

class_sz = Class_sz()

cosmo_params = {
'omega_b': 0.02242,
'omega_cdm':  0.11933,
'H0': 67.66, # use H0 because this is what is used by the emulators.
'tau_reio': 0.0561,
'ln10^{10}A_s': 3.047,
'n_s': 0.9665,  
'N_ur': 0.00641,
'N_ncdm': 3,
'm_ncdm': 0.02,
}

class_sz.set(cosmo_params)

class_sz.set({
    
'output' : 'tCl,lCl,pCl',

'cosmo_model': 4,
'skip_cmb': 0,  

        })
        
# import time
# start_time = time.time()
# class_sz.compute_class_szfast()
# end_time = time.time()
# elapsed_time = end_time - start_time
import timeit

# Define a function that executes the method
def timed_execution():
    class_sz.compute_class_szfast()

# Measure the execution time
# Number of evaluations
num_evaluations = 10
execution_time = timeit.timeit(timed_execution, number=num_evaluations)

print(f"Execution time: {execution_time/num_evaluations} seconds")


lensed_cls = class_sz.lensed_cl()
l_fast = lensed_cls['ell']
cl_tt_fast = lensed_cls['tt']
cl_ee_fast = lensed_cls['ee']
cl_te_fast = lensed_cls['te']
cl_pp_fast = lensed_cls['pp']



# a simple conversion from cl's to dl's
def l_to_dl(lp):
    return lp*(lp+1.)/2./np.pi



label_size = 15
title_size = 20
legend_size = 13
handle_length = 1.5
fig, (ax1,ax2,ax3) = plt.subplots(1,3,figsize=(18,5))


ax = ax1
ax.tick_params(axis = 'x',which='both',length=5,direction='in', pad=10)
ax.tick_params(axis = 'y',which='both',length=5,direction='in', pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
plt.setp(ax.get_yticklabels(), rotation='horizontal', fontsize=label_size)
plt.setp(ax.get_xticklabels(), fontsize=label_size)
ax.grid( visible=True, which="both", alpha=0.2, linestyle='--')

ax.plot(l_fast,l_to_dl(l_fast)*cl_ee_fast,ls='-.',c='red',label='class_szfast')



ax.set_ylabel(r"$D_\ell$",size=title_size)
ax.set_xlabel(r"$\ell$",size=title_size)
ax.set_xscale('log')
ax.set_yscale('log')
# ax.set_ylim(2e0,1e4)
ax.set_xlim(2,1e4)
ax.legend(fontsize=legend_size)


ax = ax2
ax.tick_params(axis = 'x',which='both',length=5,direction='in', pad=10)
ax.tick_params(axis = 'y',which='both',length=5,direction='in', pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
plt.setp(ax.get_yticklabels(), rotation='horizontal', fontsize=label_size)
plt.setp(ax.get_xticklabels(), fontsize=label_size)
ax.grid( visible=True, which="both", alpha=0.2, linestyle='--')


ax.plot(l_fast,l_to_dl(l_fast)*cl_te_fast,ls='-.',label='class_szfast')


ax.set_ylabel(r"$D_\ell^\mathrm{TE}$",size=title_size)
ax.set_xlabel(r"$\ell$",size=title_size)
ax.set_xscale('log')
# ax.set_yscale('log')
# ax.set_ylim(2e0,1e4)
ax.set_xlim(2,1e4)
ax.legend(fontsize=legend_size)


ax = ax3
ax.tick_params(axis = 'x',which='both',length=5,direction='in', pad=10)
ax.tick_params(axis = 'y',which='both',length=5,direction='in', pad=5)
ax.xaxis.set_ticks_position('both')
ax.yaxis.set_ticks_position('both')
plt.setp(ax.get_yticklabels(), rotation='horizontal', fontsize=label_size)
plt.setp(ax.get_xticklabels(), fontsize=label_size)
ax.grid( visible=True, which="both", alpha=0.2, linestyle='--')


ax.plot(l_fast,l_to_dl(l_fast)*cl_pp_fast*(l_fast*(l_fast+1.)),ls='-.',label='class_szfast')

ax.set_ylabel(r"$[\ell(\ell+1)]^2 C_\ell^{\phi\phi}/2\pi$",size=title_size)
ax.set_xlabel(r"$\ell$",size=title_size)
ax.set_xscale('log')
ax.set_yscale('log')
# ax.set_ylim(2e0,1e4)
ax.set_xlim(2,1e4)
ax.legend(fontsize=legend_size)
fig.tight_layout()

plt.show()



import numpy as np
from classy_sz import Class as Class_sz
import os
import time


cosmo_params_with_sigma_8 = {
'Omega_b': 0.05,
'omega_c':  0.11933,
'H0': 67.66, # use H0 because this is what is used by the emulators and to avoid any ambiguity when comparing with camb. 
'tau_reio': 0.0561,
'sigma8': 0.8118,
'n_s': 0.9665,
'm_ncdm': 0.02,

}


classy_sz = Class_sz()
classy_sz.set(cosmo_params_with_sigma_8)
classy_sz.set({
'output':'tCl,lCl,pCl',
'skip_chi':0,
'skip_hubble':0,     
'cosmo_model': 1   
})


start_time = time.time()
classy_sz.compute_class_szfast()
print("Execution time: {:.5f} seconds".format(time.time() - start_time))

# print('sigma8:',classy_sz.sigma8())
# print('A_s:',classy_sz.get_current_derived_parameters(['A_s']))
# print('m_ncdm_in_eV:',classy_sz.get_current_derived_parameters(['m_ncdm_in_eV']))

print('DA',classy_sz.angular_distance(1.))
print('Hubble',classy_sz.Hubble(1.))


classy_sz = Class_sz()
classy_sz.set(cosmo_params_with_sigma_8)
classy_sz.set({
'output':'tCl,lCl,pCl',
'skip_chi':0,
'skip_hubble':0,     
'cosmo_model': 6   
})
start_time = time.time()
classy_sz.compute_class_szfast()
print("Execution time: {:.5f} seconds".format(time.time() - start_time))

print('DA',classy_sz.angular_distance(1.))
print('Hubble',classy_sz.Hubble(1.))


from classy_sz import Class
Mclass_sz = Class()
class_sz_cosmo_params = {
 # 'Omega_b': 0.049,
 'omega_b': 0.049*(67.11/100.)**2,
 'Omega_cdm': 0.2685,
 'H0': 67.11,
 'sigma8': 0.834,
 'n_s': 0.9624}


# Mclass_sz.set(class_sz_cosmo_params) 
Mclass_sz.set({


'output': 'tSZ_1h,tSZ_2h',
'non_linear':'hmcode',
'pressure profile':'B12',
'delta for electron pressure':'200c',
"concentration parameter":"D08",
"ell_min" : 2,
"ell_max" : 9725,
'dell': 0,
'dlogell': 0.1,
    
'M_min' : 4.9e11, # got from will on 24/05/24
'M_max' : 2.2e15,# got from will on 24/05/24

'z_min': 0.03,
'z_max': 4.,# got from will on 24/05/24
    
    
# 'n_z_pressure_profile': 500,
# 'n_m_pressure_profile' : 500,
# 'n_l_pressure_profile' : 500,
    
'l_min_gas_pressure_profile' :  1.e-2,
'l_max_gas_pressure_profile' :  5.e4,    

'pressure_profile_epsrel':1e-4,
'pressure_profile_epsabs':1e-100,
    

    
    
'hm_consistency' : 0,
    

'use_fft_for_profiles_transform' : 1,
'x_min_gas_pressure_fftw' : 1e-5,
'x_max_gas_pressure_fftw' : 1e5,
'N_samp_fftw' : 8192,
    
    
# 'ndim_masses' : 500,
# 'ndim_redshifts' :100,
'redshift_epsrel': 1e-6,
'redshift_epsabs': 1e-100,
'mass_epsrel':1e-6,
'mass_epsabs':1e-100,    
    

'truncate_gas_pressure_wrt_rvir' : 1,
'x_outSZ': 2.,
'mass function' : 'T08M200c',
# 'T10_alpha_fixed' : 1,
    
    
'P_k_max_h/Mpc': 10.,
'k_per_decade_class_sz':80.,
'k_min_for_pk_class_sz':1e-4,
'k_max_for_pk_class_sz': 10*0.7, 

})
Mclass_sz.compute()
# Mclass_sz.compute_class_szfast()

cl_sz = Mclass_sz.cl_sz()
print(cl_sz)



from classy_sz import Class
import numpy as np

Omegam0 = 0.3075
H0 = 67.74
Omegab = 0.0486
Omegac = 0.2589
Omegac + Omegab
omega_b = Omegab*(H0/100.)**2
omega_c = Omegac*(H0/100.)**2
hparam = H0/100.
maniyar_cosmo = {
'omega_b': omega_b,
'omega_cdm':  omega_c,
'h': H0/100.,
# 'tau_reio': 0.0561,
'ln10^{10}A_s': 3.048,
'n_s': 0.9665,
# 'sigma8':0.830,
'k_pivot': 0.05,
'N_ncdm': 1,
'N_ur': 2.0328,
'm_ncdm': 0.0
}

M = Class()
M.set({'output':'dndlnM,cib_cib_1h,cib_cib_2h,cib_monopole'})
M.set(maniyar_cosmo)
M.set({

'mass function' : 'T08M200c',
'use_maniyar_cib_model':1,

'maniyar_cib_etamax' : 5.12572945e-01,

'maniyar_cib_zc' : 1.5,
'maniyar_cib_tau' : 8.25475287e-01,
'maniyar_cib_fsub' : 0.134*np.log(10.),
'Most_efficient_halo_mass_in_Msun' : 5.34372069e+12,
'Size_of_halo_masses_sourcing_CIB_emission' :  1.5583436676980493,
#for the Lsat tabulation:
'freq_min': 9e1,
'freq_max': 1.5e3,
'dlogfreq' : 0.1,

'concentration parameter':'fixed', # this sets it to 5

'n_z_L_sat' :100,
'n_m_L_sat' :100,
'n_nu_L_sat':500,

'use_nc_1_for_all_halos_cib_HOD': 1,

'sub_halo_mass_function' : 'TW10',#'JvdB14',
'M_min_subhalo_in_Msun' : 1e5, # 1e5 see https://github.com/abhimaniyar/halomodel_cib_tsz_cibxtsz/blob/master/Cell_cib.py
'use_redshift_dependent_M_min': 0,
'M_min' : 1e8*hparam,
'M_max' : 1e15*hparam,
'z_min' : 0.012,
'z_max' : 10.,
'ell_min': 10.,
'ell_max':5e4,
'dlogell':0.3,


'ndim_redshifts': 210,
'ndim_masses':150,

'has_cib_flux_cut': 0,
'hm_consistency':0,

'epsabs_L_sat': 1e-40,
'epsrel_L_sat': 1e-9,
    
'damping_1h_term':0,

# "P_k_max_1/Mpc": 50.,
# 'k_max_for_pk_class_sz':50.
})

M.set({
       'cib_frequency_list_num' : 2,
       'cib_frequency_list_in_GHz' : '100,240',
       'class_sz_verbose': 10,
      })
M.compute()




import classy_szfast
import classy_sz
from classy_sz import Class as Class_sz

path_to_class_sz = "/Users/boris/Work/CLASS-SZ/SO-SZ/class_sz/class-sz/"
path_to_class_sz_files =  path_to_class_sz+"class_sz_auxiliary_files/excludes/galaxies/"

cosmo_params = {
'omega_b': 0.022,
'omega_cdm':  0.122,
'H0': 67.5, 
'tau_reio': 0.0561,
'ln10^{10}A_s': 3.047,
'n_s': 0.965,
'k_pivot': 0.05, 
'N_ncdm': 1,
'N_ur': 2.0328,
'm_ncdm': 0.06 
}

class_sz = Class_sz()

class_sz.set(cosmo_params)



common_params = {

# here gal_gal_1h and gal_gal_2h correspond to the HOD calculation
# while gal_gal_hf is the linear bias calculation
# dndlnm ensures we can access the halo-function after computation
'output': 'gal_gal_1h,gal_gal_2h,gal_gal_hf,dndlnm',    

'galaxy_sample' : 'custom', # custom made galaxy sample
'full_path_to_dndz_gal':path_to_class_sz_files+'/dndz_quaia.txt', # redshift distribution of custom made galaxy sample

# pick redshift range
'z_min' : 0.005,
'z_max' : 3.0,
    
'has_b_custom1': 1, # do you want a cutom function for b(z)? yes: 1, no: 0
'array_b_custom1_n_z': 100, # precision parameters for z sampling of b(z)    
'non_linear' : 'hmcode', # non-linear model for matter P(k), e.g., 'hmcode' or 'halofit'
'effective_galaxy_bias': 2.,
# for the custom bias function we want: 
# b(z) = bquaia(z,bl) as defined above
# We code this in path_to_class_sz/python/classy_szfast/classy_szfast/custom_bias/custom_bias.py
    
    
# pick mass range (relevant for HOD prediction)
'M_min' : 1e11, 
'M_max' : 1e15,
    

    
# Other parameters relevant for HOD predictions
'mass function' : 'T08M200c', # halo mass function, e.g., Tinker et al 2008 defined at M200c
'concentration parameter' : 'B13', # halo concentration parameter  

# main HOD parameters controlling Nc, Ns
'sigma_log10M_HOD': 0.68,
'alpha_s_HOD':    1.30,
'M1_prime_HOD': 10**12.7, # msun/h
'M_min_HOD': 10**11.8, # msun/h
'M0_HOD' :0,
    
# these are not important (maybe for fine-tuning HOD)
'x_out_truncated_nfw_profile_satellite_galaxies':  1.,
'f_cen_HOD' : 1., 
'hm_consistency' : 0,
    
}

# if you need other cosmological parameters
# do: 
cosmo_params.update(
{
'omega_cdm':0.11,
}
)

class_sz.set(common_params)

class_sz.compute_class_szfast()




import os
import sys
from setuptools import setup, find_packages, Extension
from Cython.Distutils import build_ext
import numpy as np
import site
import subprocess as sbp

# Get the site-packages path
path_install = site.getusersitepackages() if os.path.exists(site.getusersitepackages()) else site.getsitepackages()[0]
path_install = os.path.join(path_install, "class-sz")

# Helper function to include additional package files
def package_files(directory, exclude_dirs):
    paths = []
    for (path, directories, filenames) in os.walk(directory):
        directories[:] = [d for d in directories if os.path.join(path, d) not in exclude_dirs]
        for filename in filenames:
            paths.append(os.path.relpath(os.path.join(path, filename), directory))
    return paths

exclude_dirs = ['class-sz/output', 'class-sz/test', 'class-sz/class_sz_auxiliary_files/excludes']
pck_files = package_files('class-sz', exclude_dirs)

# Get the GCC compiler path
GCCPATH_STRING = sbp.Popen(['gcc', '-print-libgcc-file-name'], stdout=sbp.PIPE).communicate()[0]
GCCPATH = os.path.normpath(os.path.dirname(GCCPATH_STRING)).decode()

# Determine libraries to link
liblist = ["class"]
MVEC_STRING = sbp.Popen(['gcc', '-lmvec'], stderr=sbp.PIPE).communicate()[1]
if b"mvec" not in MVEC_STRING:
    liblist += ["mvec", "m"]

# Define Cython extension
classy_sz_ext = Extension(
    "classy_sz",
    [os.path.join("class-sz/python", "classy.pyx")],
    include_dirs=[np.get_include(), os.path.join("class-sz", "include")],
    libraries=liblist,
    library_dirs=["class-sz", GCCPATH],
    extra_link_args=['-lgomp', '-lgsl', '-lfftw3', '-lgslcblas']
)

classy_sz_ext.cython_directives = {'language_level': "3" if sys.version_info.major >= 3 else "2"}

# Custom build_ext class to handle building `libclass.a`
class ClassyBuildExt(build_ext):
    def run(self):
        run_env = dict(CLASSDIR=path_install, **os.environ.copy())
        # Print the current working directory for debugging purposes
        print("Current working directory:", os.getcwd())

        # Change permissions on the select_makefile.sh script
        print("Running chmod to make select_makefile.sh executable")
        sbp.run(["chmod", "+x", "select_makefile.sh"], cwd=os.path.join(os.getcwd(), "class-sz"), env=run_env, check=True)

        # Run the script to select the correct Makefile
        print("Running the script to select the correct Makefile")
        sbp.run(["./select_makefile.sh"], cwd=os.path.join(os.getcwd(), "class-sz"), env=run_env, check=True)

        print("Building the library") ## prints wont print unless -vvv is used
        result = sbp.run("make libclass.a -j", shell=True, cwd=os.path.join(os.getcwd(), "class-sz"), env=run_env)
        if result.returncode != 0:
            raise RuntimeError("Building libclass.a failed")

        build_ext.run(self)

print(pck_files)
# Setup function
setup(
    name='classy_sz',
    version="0.1.7",
    author="Boris Bolliet, Ola Kusiak",
    author_email="bb667@cam.ac.uk, akk2175@columbia.edu",
    description='CLASS-SZ in Python',
    long_description="See ('https://github.com/CLASS-SZ').",
    url='https://github.com/CLASS-SZ',
    cmdclass={'build_ext': ClassyBuildExt},
    ext_modules=[classy_sz_ext],
    packages=find_packages(where='class-sz/python'),
    package_dir={'': 'class-sz/python'},
    package_data={'class-sz': pck_files},
    include_package_data=True,
)



from classy_sz import Class
M = Class()
cosmo_params = {
'omega_b': 0.02242,
'omega_cdm':  0.11933,
'H0': 67.66, # use H0 because this is what is used by the emulators.
'tau_reio': 0.0561,
'ln10^{10}A_s': 3.047,
'n_s': 0.9665,

# 'k_pivot': 0.05,
# 'N_ncdm': 1,
# 'N_ur': 2.0328,
# 'm_ncdm': 0.06    

}

M = Class()
# M.set(cosmo_params)
M.set({
'output' : 'gal_gal_1h',#,galn_lens_1h,galn_lens_2h',
# 'output' : 'gal_lens_1h,gal_lens_2h',
# 'galaxy_samples_list_num' : 1, # the number of galaxy samples
# 'galaxy_samples_list' : '1', # the id string of each sample, can be any integer
# 'full_path_and_prefix_to_dndz_ngal':'/Users/boris/Work/CLASS-SZ/SO-SZ/class_sz/class_sz_auxiliary_files/WISC_bin3_ngal_example',
'full_path_to_dndz_gal':'/Users/boris/Work/CLASS-SZ/SO-SZ/class_sz/class-sz/class_sz_auxiliary_files/includes/WISC_bin3_ngal_example1.txt',
'galaxy_sample' : 'custom'
})
M.set({# class_sz parameters:

'mass function': 'T08M200c',
'concentration parameter' : 'D08',
'hm_consistency': 0,

'delta for galaxies' : '200c',
'delta for matter density' : '200c',
    
'x_out_truncated_nfw_profile' : 1,

# HOD parameters
'sigma_log10M_HOD' : 0.69,
'alpha_s_HOD' :  0.,
'M1_prime_HOD' : 5.03e12, # Msun/h
'M_min_HOD' : 6.25e11, # Msun/h
'M0_HOD' : 0,  # Msun/h ## set to very high value so Ns always 0.
'x_out_truncated_nfw_profile_satellite_galaxies' : 1.09,    
'f_cen_HOD' : 1., 
    
    
# # # HOD parameters
# 'sigma_log10M_HOD_ngal_0' : 0.69,
# 'alpha_s_HOD_ngal_0' :  0.,
# 'M1_prime_HOD_ngal_0' : 5.03e12, # Msun/h
# 'M_min_HOD_ngal_0' : 6.25e11, # Msun/h
# 'M0_HOD_ngal_0' : 6.25e11,  # Msun/h
# 'x_out_truncated_nfw_profile_satellite_galaxies_ngal_0' : 1.09,    
# 'f_cen_HOD_ngal_0' : 1., 
    

'M0 equal M_min (HOD)'  : 'no',



'M_min' : 1e11,
'M_max' : 5e15,
'ndim_masses' : 100,
    
    
'z_min' : 1e-3,
'z_max' : 3.,
'ndim_redshifts' : 100,
    
    
'dlogell' : 0.3,
'ell_max' : 20000.0,
'ell_min' : 2.0,


    
# precisions params:
'non_linear' : 'halofit',
'k_min_for_pk_class_sz' :  0.001,
'k_max_for_pk_class_sz' :  60.0,
'k_per_decade_class_sz' :  50,
'P_k_max_h/Mpc' :  50.0,

    
'redshift_epsabs' : 1e-40,
'redshift_epsrel' : 0.0001,
'mass_epsabs' : 1e-40,
'mass_epsrel' : 0.0001,    
'class_sz_verbose' : 12,

'cosmo_model' : 1,
        })
        
# M.compute_class_szfast()
M.compute()
cl_galn_lens = M.cl_galn_lens()
cl_gal_gal = M.cl_gg()

print(cl_gal_gal)
